package de.k621.avr.gps.converter;

/**
 * Ein Modell eines NMEA-Kommandos des Typs 'GPGGA'.
 *
 * @author Martin Matysiak
 */
public class KGGAMessage {

  private String rawTime = "";
  private String rawLat = "0000.0000",  rawLon = "0000.0000";
  private char latIndicator = '0',  lonIndicator = '0';
  private int fix = 0,  sat = 0;
  private float alt = 0,  hdop = 0;

  /**
   * Ausgabeformat : Grad
   */
  public static final int DEGREES = 2;
  /**
   * Ausgabeformat : Bogenmaß
   */
  public static final int RADIANS = 3;

  /**
   * Konstruktor der Klasse
   *
   * @param rawString Das NMEA-Kommando, das abgebildet werden soll
   */
  public KGGAMessage(String rawString) {
    String[] rawData = rawString.split(",");

    rawTime = rawData[1];
    if (!rawData[2].isEmpty()) {
      rawLat = rawData[2];
    }

    if (!rawData[3].isEmpty()) {
      latIndicator = rawData[3].charAt(0);
    }

    if (!rawData[4].isEmpty()) {
      rawLon = rawData[4];
    }

    if (!rawData[5].isEmpty()) {
      lonIndicator = rawData[5].charAt(0);
    }

    try {
      fix = Integer.parseInt(rawData[6]);
    } catch (Exception ex) {
      fix = 0;
    }
    try {
      sat = Integer.parseInt(rawData[7]);
    } catch (Exception ex) {
      sat = 0;
    }
    try {
      hdop = Float.parseFloat(rawData[8]);
    } catch (Exception ex) {
      hdop = 0;
    }
    try {
      alt = Float.parseFloat(rawData[9]);
    } catch (Exception ex) {
      alt = 0;
    }
  }

  /**
   * Gibt Breitengrad zurück
   *
   * @param format Das Rückgabeformat
   * @return Den Breitengrad im gewünschten Format
   */
  public float getLatitude(int format) {
    float output = 0;
    switch (format) {
      case DEGREES:
        output = Float.parseFloat(rawLat.substring(0, 2));
        output += Float.parseFloat(rawLat.substring(2)) / 60;
        if (latIndicator == 'S') {
          output *= -1;
        }
        return output;
      case RADIANS:
        return getLatitude(DEGREES) * ((float) Math.PI / 180F);
      default:
        return getLatitude(DEGREES);
    }
  }

  /**
   * Gibt Längengrad zurück
   *
   * @param format Das Rückgabeformat
   * @return Den Längengrad im gewünschten Format
   */
  public float getLongitude(int format) {
    float output = 0;
    switch (format) {
      case DEGREES:
        output = Float.parseFloat(rawLon.substring(0, 3));
        output += Float.parseFloat(rawLon.substring(3)) / 60;
        if (lonIndicator == 'W') {
          output *= -1;
        }
        return output;
      case RADIANS:
        return getLongitude(DEGREES) * ((float) Math.PI / 180F);
      default:
        return getLongitude(DEGREES);
    }
  }

  /**
   * @return Die Höhenmeter
   */
  public float getAltitude() {
    return alt;
  }

  /**
   * @return Die Zeit
   */
  public String getTime() {
    return rawTime.substring(0, 2) + ":" + rawTime.substring(2, 4) + ":" + rawTime.substring(4, 6);
  }

  /**
   * @return true wenn Kommando valide, ansonsten false
   */
  public boolean isValid() {
    return fix >= 1 ? true : false;
  }

  /**
   * @param count Die Mindestanzahl an Satteliten
   * @return true wenn Anzahl an Satteliten größer oder gleich der Mindestanzahl ist, ansonsten false
   */
  public boolean hasSattelites(int count) {
    return sat >= count ? true : false;
  }

  /**
   * @param inHdop Der maximal zulässige HDOP Wert
   * @return true wenn HDOP kleiner oder gleich dem Maximalwert ist, ansonsten false
   */
  public boolean hasLowerHdopThan(float inHdop) {
    return hdop <= inHdop ? true : false;
  }
}