package de.k621.avr.gps.converter;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Die Konvertierungsklasse für GPX-Dateien.
 *
 * @author Martin Matysiak
 */
public class KGPXConverter extends KConverter {

  private int fMinimumSattelites = 0;
  private float fMaximumHDOP = Float.POSITIVE_INFINITY;

  /**
   * Konstruktor der Klasse
   *
   * @param pInput Die Eingabedatei
   * @param pOutput Die Ausgabedatei
   * @param pInterface Das Statusinterface
   */
  public KGPXConverter(BufferedReader pInput, BufferedWriter pOutput,
          KProgressViewer pInterface) {
    super(pInput, pOutput, pInterface);
  }

  /**
   * Konstruktor der Klasse
   *
   * @param pInput Die Eingabedatei
   * @param pOutput Die Ausgabedatei
   * @param pInterface Das Statusinterface
   * @param pMinSattelites Die minimale Anzahl an Satelliten, die eine Nachricht besitzen muss
   * @param pMaxHDOP Der maximale HDOP Wert, den eine Nachricht besitzen darf
   */
  public KGPXConverter(BufferedReader pInput, BufferedWriter pOutput,
          KProgressViewer pInterface, int pMinSattelites, float pMaxHDOP) {

    super(pInput, pOutput, pInterface);

    fMinimumSattelites = pMinSattelites;
    fMaximumHDOP = pMaxHDOP;

  }

  @Override
  public void run() {
    fInterface.updateText("Bereite Konvertierung vor...");
    fInterface.setBarMax(2); //2 Schritte: Einlesen, Schreiben

    //Punkte einlesen
    Vector<KGGAMessage> outPoints = new Vector<KGGAMessage>();
    String creationDate = "";

    try {
      String inLine = fInput.readLine();
      boolean foundDate = false;

      while (inLine != null && !inLine.isEmpty()) {

        //Nicht GGA-Meldungen ignorieren
        if (inLine.startsWith("$GPGGA") && isValidMsg(inLine)) {
          KGGAMessage message = new KGGAMessage(inLine);

          if (message.isValid() &&
                  message.hasSattelites(fMinimumSattelites) &&
                  message.hasLowerHdopThan(fMaximumHDOP)) {
            outPoints.add(message);
          }

        }

        //Einmalig nach valider RMC Nachricht suchen, um Datum zu ermitteln
        if (!foundDate && (inLine.startsWith("$GPRMC") && isValidMsg(inLine) && inLine.contains(",A,"))) {
          String[] splitLine = inLine.split(",");
          creationDate = "20" + //Bitte alle 100 Jahre umstellen
                  splitLine[9].substring(4, 6) + "-" +
                  splitLine[9].substring(2, 4) + "-" +
                  splitLine[9].substring(0, 2);
        }

        inLine = fInput.readLine();
      }
    } catch (IOException ioex) {
      javax.swing.JOptionPane.showMessageDialog(null,
              ioex.getLocalizedMessage(),
              "Fehler",
              javax.swing.JOptionPane.ERROR_MESSAGE);
    }

    fInterface.updateText("Schreibe Datei...");
    fInterface.updateBar(1);

    try {

      //Datum ermitteln
      Date dt = new Date();
      // Festlegung des Formats:
      SimpleDateFormat df = new SimpleDateFormat("dd.MM.yyyy");
      SimpleDateFormat year = new SimpleDateFormat("yyyy");
      SimpleDateFormat dateTime = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ");
      String date = (new SimpleDateFormat("yyyy-MM-dd")).format(dt);
      String timeZone = (new SimpleDateFormat("Z")).format(dt);

      //Header schreiben
      fOutput.write("<?xml version=\"1.\" encoding=\"ISO-8859-1\" standalone=\"yes\"?>\n" +
              "<gpx version=\"1.1\" creator=\"NMEA-Toolbox (k621.de)\"\n" +
              "  xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"  xmlns:topografix=\"http://www.topografix.com/GPX/Private/TopoGrafix/0/1\"\n" +
              "  xmlns=\"http://www.topografix.com/GPX/1/1\"\n" +
              "  xsi:schemaLocation=\"http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd\">\n" +
              "\t<metadata>\n" +
              "\t\t<name>GPS Log</name>\n" +
              "\t\t<desc>Erstellt am " + df.format(dt) + "</desc>\n" +
              "\t\t<author>Martin Matysiak (k621.de)</author>\n" +
              "\t\t<copyright author=\"Martin Matysiak\">\n" +
              "\t\t\t<year>" + year.format(dt) + "</year>\n" +
              "\t\t\t<license>http://creativecommons.org/licenses/by-nc-sa/3.0/de/</license>\n" +
              "\t\t</copyright>\n" +
              "\t\t<link href=\"http://www.k621.de\">\n" +
              "\t\t\t<text>Homepage des Autors</text>\n" +
              "\t\t\t<type>text/html</type>\n" +
              "\t\t</link>\n" +
              "\t\t<time>" + dateTime.format(dt) + "</time>\n" +
              "\t</metadata>\n" +
              "\t<trk>\n" +
              "\t\t<name>Aufgezeichneter GPS Pfad</name>\n" +
              "\t\t<trkseg>\n");

      //Vektor in Datei schreiben
      for (int i = 0; i < outPoints.size(); i++) {



        fOutput.write("\t\t\t<trkpt lat=\"" + outPoints.get(i).getLatitude(KGGAMessage.DEGREES) + "\" lon=\"" + outPoints.get(i).getLongitude(KGGAMessage.DEGREES) + "\">\n" +
                "\t\t\t\t<ele>" + outPoints.get(i).getAltitude() + "</ele>\n" +
                "\t\t\t\t<time>" + creationDate + "T" + outPoints.get(i).getTime() + timeZone + "</time>\n" +
                "\t\t\t</trkpt>\n");
      }

      //Footer schreiben
      fOutput.write("\t\t</trkseg>\n" +
              "\t</trk>\n" +
              "</gpx>");

      fOutput.close();

      fInterface.updateText("Datei erfolgreich erstellt!");
      fInterface.updateBar(-1);

    } catch (IOException ioex) {
      javax.swing.JOptionPane.showMessageDialog(null,
              ioex.getLocalizedMessage(),
              "Fehler",
              javax.swing.JOptionPane.ERROR_MESSAGE);
    } finally {
      try {
        fInput.close();
        fOutput.close();
      } catch (IOException ioex) {
        //Nichts tun
      }
    }
  }
}
