package de.k621.avr.gps.converter;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Die Konvertierungsklasse für KML-Dateien. Diese können in Google Earth sowie
 * Google Maps betrachtet werden.
 *
 * @author Martin Matysiak
 */
public class KKMLConverter extends KConverter {

  private int fMinimumSattelites = 0;
  private float fMaximumHDOP = Float.POSITIVE_INFINITY;

  /**
   * Konstruktor der Klasse
   *
   * @param pInput Die Eingabedatei
   * @param pOutput Die Ausgabedatei
   * @param pInterface Das Statusinterface
   */
  public KKMLConverter(BufferedReader pInput, BufferedWriter pOutput,
          KProgressViewer pInterface) {
    super(pInput, pOutput, pInterface);
  }

  /**
   * Konstruktor der Klasse
   *
   * @param pInput Die Eingabedatei
   * @param pOutput Die Ausgabedatei
   * @param pInterface Das Statusinterface
   * @param pMinSattelites Die minimale Anzahl an Satelliten, die eine Nachricht besitzen muss
   * @param pMaxHDOP Der maximale HDOP Wert, den eine Nachricht besitzen darf
   */
  public KKMLConverter(BufferedReader pInput, BufferedWriter pOutput,
          KProgressViewer pInterface, int pMinSattelites, float pMaxHDOP) {

    super(pInput, pOutput, pInterface);

    fMinimumSattelites = pMinSattelites;
    fMaximumHDOP = pMaxHDOP;

  }

  @Override
  public void run() {
    fInterface.updateText("Bereite Konvertierung vor...");
    fInterface.setBarMax(3); //3 Schritte: Einlesen, Verarbeiten, Schreiben

    //Punkte einlesen
    Vector<KGGAMessage> outPoints = new Vector<KGGAMessage>();

    try {
      String inLine = fInput.readLine();

      while (inLine != null && !inLine.isEmpty()) {

        //Nicht GGA-Meldungen ignorieren
        if (inLine.startsWith("$GPGGA") && isValidMsg(inLine)) {
          KGGAMessage message = new KGGAMessage(inLine);

          if (message.isValid() &&
                  message.hasSattelites(fMinimumSattelites) &&
                  message.hasLowerHdopThan(fMaximumHDOP)) {
            outPoints.add(message);
          }

        }

        inLine = fInput.readLine();
      }
    } catch (IOException ioex) {
      javax.swing.JOptionPane.showMessageDialog(null,
              ioex.getLocalizedMessage(),
              "Fehler",
              javax.swing.JOptionPane.ERROR_MESSAGE);
    }

    fInterface.updateText("Warte auf Benutzereingabe...");
    fInterface.updateBar(1);

    //Google Maps Filter
    if ((outPoints.size() > 350) &&
            (javax.swing.JOptionPane.showConfirmDialog(null,
            "Soll die KML-Datei Google Maps kompatibel sein? " +
            "Dabei wird die Anzahl der Wegpunkte auf 350 reduziert " +
            "(Genauigkeitsverlust!).",
            "Google Maps Kompatibilität?",
            javax.swing.JOptionPane.YES_NO_OPTION,
            javax.swing.JOptionPane.QUESTION_MESSAGE) == javax.swing.JOptionPane.YES_OPTION)) {

      int pointsToReduce = outPoints.size() - 350;

      fInterface.setBarMax(pointsToReduce + pointsToReduce / 3);
      fInterface.updateBar(pointsToReduce / 3);
      fInterface.updateText("Reduziere Anzahl der Wegpunkte...");


      //Zunächst Positionen ohne großartige Bewegung (max. 1 Meter Abweichung)
      //suchen und auf einen Punkt reduzieren


      double treshold = 1 * Math.pow(10, -8);
      int i = 0;

      while (i < outPoints.size() - 2) {
        if ((Math.pow(outPoints.get(i).getLatitude(KGGAMessage.DEGREES) -
                outPoints.get(i + 1).getLatitude(KGGAMessage.DEGREES), 2) +
                Math.pow(outPoints.get(i).getLongitude(KGGAMessage.DEGREES) -
                outPoints.get(i + 1).getLongitude(KGGAMessage.DEGREES), 2)) < treshold) {
          outPoints.remove(i + 1);
          fInterface.incrementBar();
          pointsToReduce--;
        } else {
          i++;
        }
      }

      //Anschließend gerade Strecken weitestgehend vereinfachen

      treshold = 1 * Math.pow(10, -2);
      float lat1, lat2, lat3, lon1, lon2, lon3;

      i = 1;

      lat1 = outPoints.get(0).getLatitude(KGGAMessage.DEGREES);
      lat2 = outPoints.get(1).getLatitude(KGGAMessage.DEGREES);
      lat3 = outPoints.get(2).getLatitude(KGGAMessage.DEGREES);

      lon1 = outPoints.get(0).getLongitude(KGGAMessage.DEGREES);
      lon2 = outPoints.get(1).getLongitude(KGGAMessage.DEGREES);
      lon3 = outPoints.get(2).getLongitude(KGGAMessage.DEGREES);

      while ((pointsToReduce > 0) && (i < outPoints.size() - 3)) {

        //Ausrechnen eines "Steigungsdreieckes" zwei aufeinanderfolgender Koordinaten
        //Sind die Steigungen ähnlich, mittleren Punkt verwerfen
        if ((lat2 - lat1 / lon2 - lon1) - (lat3 - lat2 / lon3 - lon2) < treshold) {
          outPoints.remove(i);
          fInterface.incrementBar();
          pointsToReduce--;
        }

        i++;

        lat1 = lat2;
        lon1 = lon2;

        lat2 = lat3;
        lon2 = lon3;

        lat3 = outPoints.get(i + 1).getLatitude(KGGAMessage.DEGREES);
        lon3 = outPoints.get(i + 1).getLongitude(KGGAMessage.DEGREES);

      }

      //Zuletzt gleichmäßig Punkte rausschmeißen, um auf die Maximalzahl zu kommen
      float step = (float) outPoints.size() / (float) pointsToReduce;
      System.out.println(outPoints.size() + " " + pointsToReduce + " " + step);
      for (i = 0; i < pointsToReduce; i++) {
        outPoints.remove((int) (Math.floor(i * step) - i));
        fInterface.incrementBar();
      }
      System.out.println(outPoints.size());


    }



    //Datei assemblieren
    fInterface.updateText("Schreibe Datei...");
    try {

      //Datum ermitteln
      Date dt = new Date();
      // Festlegung des Formats:
      SimpleDateFormat df = new SimpleDateFormat("dd.MM.yyyy");

      //Header schreiben
      fOutput.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
              "<kml xmlns=\"http://earth.google.com/kml/2.1\">\n" +
              "\t<Placemark>\n" +
              "\t\t<name>Aufgezeichneter GPS Pfad</name>\n" +
              "\t\t<description>Erstellt am " + df.format(dt) + " mit der NMEA-Toolbox :: www.k621.de</description>\n" +
              "\t\t<Style id=\"solidLine\">\n" +
              "\t\t\t<LineStyle>\n" +
              "\t\t\t\t<color>ff0060ff</color>\n" + //evtl. zufallsfarben?
              "\t\t\t\t<width>4</width>\n" +
              "\t\t\t</LineStyle>\n" +
              "\t\t</Style>\n" +
              "\t\t<styleUrl>#solidLine</styleUrl>\n" +
              "\t\t<LineString>\n" +
              "\t\t\t<extrude>0</extrude>\n" +
              "\t\t\t<tessellate>1</tessellate>\n" +
              "\t\t\t<altitudeMode>clampToGround</altitudeMode>\n" +
              "\t\t\t<coordinates>");

      //Vektor in Datei schreiben
      for (int i = 0; i < outPoints.size(); i++) {
        fOutput.write(" " + outPoints.get(i).getLongitude(KGGAMessage.DEGREES) + "," + outPoints.get(i).getLatitude(KGGAMessage.DEGREES) + "\n\t\t\t");
      }

      //Footer schreiben
      fOutput.write("</coordinates>\n" +
              "\t\t</LineString>\n" +
              "\t</Placemark>\n" +
              "</kml>");

      fOutput.close();

      fInterface.updateText("Datei erfolgreich erstellt!");
      fInterface.updateBar(-1);

    } catch (IOException ioex) {
      javax.swing.JOptionPane.showMessageDialog(null,
              ioex.getLocalizedMessage(),
              "Fehler",
              javax.swing.JOptionPane.ERROR_MESSAGE);
    } finally {
      try {
        fInput.close();
        fOutput.close();
      } catch (IOException ioex) {
        //Nichts tun
      }
    }
  }
}
